/* File name     : intdrvr.c
 * Creation date : September 1993
 * Author        : Y.Yasu, Online group, Physics department, KEK
 * Version       : 1.03
 * Modified date : August 1994, Y.Yasu
 *               : Clean up
 *               : Nov 1995, Y.Yasu
 *               : fixed bugs related to intwait routine.
 */

#define VME
#include <machine/kernel.h>
#include <machine/dvrio.h>
#include <sys/proc.h>
#include <ioctl.h>
#include <sys/file.h>
#include <machine/mem.h>
#include <sys/errno.h>
#include "intdrvr.h"
#include "intinfo.h"
#define Debug_level 10
#define printf kprintf
#define INT_ST_PRIO 30*2 + 1

static void intdrvr_int();
int int_timeout();
static int stid;

intdrvr_data *intinstall(info)
struct intdrvr_info *info;
{
  intdrvr_data *p;
  int i;

  /* Allocate memory for static information save by each device */
  p = 0;
  if( (p = (intdrvr_data *)sysbrkequiv((long)sizeof(intdrvr_data),
				       DEF_SBRK_ALIGN)) == NULL ) {
    pseterr(ENXIO);
    return((intdrvr_data *)SYSERR);
  }
  /* Set up interrupt handler */
  p->vector = &info->vector;
  p->int_info.interrupt_action = C_CALL;
  p->int_info.sem = &p->intdrvr_sem;
  p->intdrvr_sem = 0;
  intinit(p);
  p->timeout_number   = 0;
  /* Start the system thread to handle the interrupt */
  if( ( stid = ststart(intdrvr_int, MINSTK, INT_ST_PRIO, "intdrvr_int",1,
	     (char *)p ) ) == SYSERR ) {
    pseterr(EIO);
    printf("\t Coundn't start the thread\n");
    return((intdrvr_data *)SYSERR);
  }
  if( ( i = iointset(p->vector, intdrvr_int, p)) == SYSERR ) {
    printf("\t Coundn't set up interrupt handler\n");
    return((intdrvr_data *)SYSERR);
  }
  /* Return a pointer to the Driver data */
  return(p);
}

intuninstall(DrvrData)
intdrvr_data *DrvrData;
{
  iointclr(DrvrData->vector);
  stremove(stid);
  /* Deallocate memory used by static device information */
  sysfree(DrvrData, (long)sizeof(intdrvr_data));
  return(OK);
}

intopen(DrvrData, dev, fd)
intdrvr_data *DrvrData;
int dev;
struct file *fd;
{
  return(OK);
}

intclose(DrvrData, fd)
intdrvr_data *DrvrData;
struct file *fd;
{
  return(OK);
}

intread(DrvrData, fd, buf, count)
intdrvr_data *DrvrData;
struct file *fd;
char *buf;
register int count;
{
  return(0);
}

intwrite(DrvrData, fd, buf, count)
intdrvr_data *DrvrData;
struct file *fd;
char *buf;
register int count;
{
  return(count);
}

intioctl(DrvrData, fd, cmd, data)
intdrvr_data *DrvrData;
struct file *fd;
int cmd;
int *data;
{
    int intrno, vectorno;
    int status_block[2];
    int ps, status;

    switch (cmd) {
    case VMEIOC_INTINIT:
        intinit(DrvrData);
        break;

    case VMEIOC_INTOPEN:
        vectorno = data[0];
#ifdef DEBUG
        printf("INTOPEN:vector = %x\n", vectorno);
#endif
        if( vectorno < VME_MIN_VECTOR || vectorno > VME_MAX_VECTOR )
            return (EINVAL);
        intrno = vectorno - VME_MIN_VECTOR;
        DrvrData->interrupt_vector = vectorno;
#ifdef DEBUG
        printf("INTOPEN:DrvrData->interrupt_vector = %x\n",
               DrvrData->interrupt_vector);
#endif
        DrvrData->interrupt_enable = 0;
        DrvrData->interrupt_flag = 0;
        DrvrData->interrupt_count = 0;
        data[1] = intrno;
        break;

      case VMEIOC_INTCLOSE:
#ifdef DEBUG
        printf("INTCLOSE:intrno = %d\n", data[0]);
#endif
        if( data[0] < VME_INTR0 || data[0] > VME_INTR9 )
            return (EINVAL);
        intrno = data[0];
        DrvrData->interrupt_vector = 0;
        break;

    case VMEIOC_INTENABLE:
#ifdef DEBUG
        printf("INTENABLE:intrno = %d\n", data[0]);
	printf("INTENABLE:semaphore = %d\n", DrvrData->intdrvr_sem);
#endif
        if( data[0] < VME_INTR0 || data[0] > VME_INTR9 )
            return (EINVAL);
        if( !DrvrData->interrupt_vector != data[0] )
            return (ENODEV);
        intrno = data[0];
        DrvrData->interrupt_enable = 1;
        DrvrData->interrupt_flag = 0;
	/* Turn on interrupts for this particular interrupt bit */
	dvrenable( DrvrData->vector->interrupt_bit );
        break;

      case VMEIOC_INTDISABLE:
#ifdef DEBUG
        printf("INTDISABLE:intrno = %d\n", data[0]);
#endif
        if( data[0] < VME_INTR0 || data[0] > VME_INTR9 )
            return (EINVAL);
        if( !DrvrData->interrupt_vector )
            return (ENODEV);
        intrno = data[0];
        DrvrData->interrupt_enable = 0;
        DrvrData->interrupt_flag = 0;
	dvrclearint(DrvrData->vector->interrupt_bit);
        break;

      case VMEIOC_INTWAIT:
#ifdef DEBUG
        printf("INTWAIT:intrno = %d\n", data[0]);
	printf("INTWAIT:vector = %d   enable flag = %d\n",
	       DrvrData->interrupt_vector, DrvrData->interrupt_enable);
#endif
        if( data[0] < VME_INTR0 || data[0] > VME_INTR9 )
            return (EINVAL);
        if( !DrvrData->interrupt_vector )
            return (ENODEV);
        if( !DrvrData->interrupt_enable )
            return (ENODEV);
        intrno = data[0];
        /*return code =
	  -2: signal, -1:timeout, 1: success, 0:already interrupted
	*/
	DrvrData->interrupt_flag = 1;
	if( DrvrData->interrupt_count > 0 ) {
#ifdef DEBUG
          printf("INTWAIT: int. already  occurred. count = %d\n",
		 DrvrData->interrupt_count);
#endif
	  data[2] = DrvrData->interrupt_count;
	  data[3] = 0;
	  DrvrData->interrupt_flag = 0;
	  DrvrData->interrupt_count = 0;
	  return (OK);
	}
	if( data[1] == 0 )
	    DrvrData->timeout_number = 
	      timeout(int_timeout, DrvrData, VME_K_TIMEOUT*100);
	else
	    DrvrData->timeout_number = 
	      timeout(int_timeout, DrvrData, data[1]*100);
#ifdef DEBUG
	printf("INTWAIT: now wait...\n");
#endif
	status = swait(&DrvrData->intdrvr_sem, 1);
        if ( status ) {
#ifdef DEBUG
            printf("INTWAIT: signal occurred.\n");
#endif
	    disable(ps);
            cancel_timeout(DrvrData->timeout_number);
	    restore(ps);
	    data[2] = DrvrData->interrupt_count;
	    data[3] = -2;
	    DrvrData->interrupt_flag = 0;
	    DrvrData->interrupt_count = 0;
            return EINTR;
	}
        if( DrvrData->interrupt_flag == -1 ) {
#ifdef DEBUG
            printf("INTWAIT: timeout occurred.\n");
#endif
	    data[2] = DrvrData->interrupt_count;
	    data[3] = -1;
	    DrvrData->interrupt_flag = 0;
	    DrvrData->interrupt_count = 0;
            return ETIMEDOUT;
	}
	disable(ps);
	cancel_timeout(DrvrData->timeout_number);
	restore(ps);
        data[2] = DrvrData->interrupt_count;
        data[3] = DrvrData->interrupt_flag;
#ifdef DEBUG
        printf("INTWAIT:interrupt_count = %d\n", DrvrData->interrupt_count);
        printf("INTWAIT:interrupt_flag = %d\n", DrvrData->interrupt_flag);
#endif
        DrvrData->interrupt_count = 0;
        DrvrData->interrupt_flag = 0;
        break;

      case VMEIOC_INTNOWAIT:
#ifdef DEBUG
        printf("INTNOWAIT:intrno = %d\n", data[0]);
#endif
        if( data[0] < VME_INTR0 || data[0] > VME_INTR9 )
            return (EINVAL);
        if( !DrvrData->interrupt_vector )
            return (ENODEV);
        if( !DrvrData->interrupt_enable )
            return (ENODEV);
        intrno = data[0];
        data[2] = DrvrData->interrupt_count;
        data[3] = DrvrData->interrupt_flag;
        DrvrData->interrupt_flag = 0;
	DrvrData->interrupt_count = 0;
        break;

      case VMEIOC_INTCLEAR:
#ifdef DEBUG
        printf("INTCLEAR:intrno = %d\n", data[0]);
#endif
        if( data[0] < VME_INTR0 || data[0] > VME_INTR9 )
            return (EINVAL);
        if( !DrvrData->interrupt_vector )
            return (ENODEV);
        if( !DrvrData->interrupt_enable )
            return (ENODEV);
        intrno = data[0];
        DrvrData->interrupt_flag = 0;
        DrvrData->interrupt_count = 0;
	break;

      case VMEIOC_INTCHECK:
#ifdef DEBUG
        printf("INTCHECK:intrno = %d\n", data[0]);
#endif
        if( data[0] < VME_INTR0 || data[0] > VME_INTR9 )
            return (EINVAL);
        if( !DrvrData->interrupt_vector )
            return (ENODEV);
        if( !DrvrData->interrupt_enable )
            return (ENODEV);
        intrno = data[0];
        data[2] = DrvrData->interrupt_count;
        data[3] = DrvrData->interrupt_flag;
        break;
    }
    return(OK);
}

intinit(DrvrData)
intdrvr_data *DrvrData; {
    DrvrData->interrupt_vector = 0;
    DrvrData->interrupt_enable = 0;
    DrvrData->interrupt_flag = 0;
    DrvrData->interrupt_count = 0;
}

static void intdrvr_int(DrvrData)
intdrvr_data *DrvrData;
{
    intdrvr_intr(DrvrData, VME_INTR0);
}

intdrvr_intr(DrvrData, intrno )
intdrvr_data *DrvrData;
int intrno; {

    /* clear interrupt bit */
    dvrclearint(DrvrData->vector->interrupt_bit);
#ifdef DEBUG
    printf("vmeaintr: interrupt occurred. intrno = %d\n", intrno);
#endif
    if( DrvrData->interrupt_vector && DrvrData->interrupt_enable &&
        DrvrData->interrupt_flag ) {
        DrvrData->interrupt_count++;
	ssignal(&DrvrData->intdrvr_sem);
    }
    if( DrvrData->interrupt_vector && DrvrData->interrupt_enable &&
        !DrvrData->interrupt_flag ) {
        DrvrData->interrupt_count++;
    }
    dvrenable(DrvrData->vector->interrupt_bit);
}


int_timeout(DrvrData)
intdrvr_data *DrvrData; {

    DrvrData->interrupt_flag = -1; /* indicate timeout */
    ssignal(&DrvrData->intdrvr_sem);
    return (0);
}


intselect(DrvrData, fd, which, se)
intdrvr_data *DrvrData;
struct file *fd;
int which;
struct sel *se;
{
  return(OK);
}




