/* File name     : intreg_test4.c
 * Creation date : May 1993
 * Author        : Y.Yasu, Online group, Physics department, KEK
 * Description   : General Purpose VME Interrupt Handling Test
 *                 The program checks if vme_intwait routine works or not.
 *                 After vme_wait routine is called, an signal occurs before
 *                 an interrupt occurs.
 *                 The program assumes the usage of VME Interrupt & I/O
 *                 Register Module(KEK product). A least one Lemo cable
 *                 should be connected between IN and OUT in the front panel.
 */

#include <stdio.h>
#include<sys/types.h>
#include <signal.h>
#include "vmelib.h"
#include "intreg.h"
#define INTREG_VECTOR INTREG_VECTOR0

volatile struct intreg *mm;
extern errno;

int count;
void dummy()
{
    count++;
    printf("signal routine : get signal :SIGALRM count = %d\n", count );
    return;
}

main(argc, argv)
int argc;
char **argv;
{
    int i, j;
    volatile caddr_t vme_addr;
    int loop,bit;
    int status, intrno, intcount;
    int pid, pid0;
    struct sigvec vec;

/* define signal handler for SIGALRM */
    vec.sv_handler = dummy;
    vec.sv_mask = 0;
    vec.sv_flags = 0;
/*    vec.sv_flags = SV_INTERRUPT;*/
    sigvec(SIGALRM, &vec, NULL);
    count = 0;

    vme_addr = (volatile caddr_t)vme_mapopen(VME_A16D16MODE, INTREG_ADDR,
                                             INTREG_SIZE);
    if (vme_addr == (volatile caddr_t)-1 ) {
        printf("intreg cannot open! \n");
        exit(1);
    }

    mm = (volatile struct intreg *)vme_addr;

    printf("address = %x\n", mm);
    vme_mapprobe(mm, INTREG_SIZE, VME_A16D16MODE);

    /* display contents of registers... */
    printf("Latch 0 = %4X\n",mm->latch[0] & 0xFF);
    printf("Latch 1 = %4X\n",mm->latch[1] & 0xFF);
    printf("in      = %4X\n",mm->in & 0xFF);
    printf("csr 0   = %4X\n",mm->csr[0] & 0xFF);
    printf("csr 1   = %4X\n",mm->csr[1] & 0xFF);

    pid0 = getpid();
    pid = fork();
    if( pid == 0 ) { /* Child process */
        /* Generate Interrupt Pulse */
        for( i = 0; i < 10; i++ ) {
	    sleep(4);
	    printf("Child process : Generate SIGALRM\n");
            kill(pid0, SIGALRM);
	}
	exit(0);
    } else if ( pid < 0 ) { /* Error */
        printf("Can't fork\n");
	exit(1);
    }
    /* Parent process */
    /* open interrupt vector */
    if( (status = vme_intopen(INTREG_VECTOR, &intrno)) != 0 ) {
        printf("errno = %d\n", status );
	exit(0);
    }
    printf("intrno = %d\n", intrno);
    printf("vme_intopen -- intrno = %d\n", intrno);
    printf("vme_intopen done\n");
    /* enable the vector */
    if( (status = vme_intenable(intrno) ) != 0 ) {
        printf("errno = %d\n", status );
        exit(0);
    }
    printf("vme_intenable done\n");
    /* wait for an interrupt */
    printf("interrupt count = %d\n", intcount);

    /* Generate Interrupt Pulse */
    for( i = 0; i < 10; i++ ) {
	if( (status = vme_intwait(intrno, &intcount, 10)) != 0 ) {
	    printf("errno = %d\n", status );
	}
	errno = 0;
	printf("Parent process : intcount = %d\n", intcount);
	printf("Parent process : CSR0 = %3X  CSR1 = %3X\n",
	       mm->csr[0]&0x0FF,mm->csr[1]&0x0FF);
    }

    /* close the vector */
    vme_intclose(intrno);
}


