/* File name     : vmeint_measure1.c
 * Creation date : March 1994
 * Author        : Y.Yasu, Online group, Physics department, KEK
 * Description   : General Purpose VME Interrupt Handling Test
 *                 The program measures the time between the occurence of
 *                 interrupt from VMEbus and the first execution of user's
 *                 program which issues vme_intwait.
 *                 The program assumes the usage of VME Interrupt & I/O
 *                 Register Module(KEK product). A least one Lemo cable
 *                 should be connected between IN and OUT in the front panel.
 */

#include <stdio.h>
#include<sys/types.h>
#include <signal.h>
#include <time.h>
#include <sys/ipc.h>
#include <sys/errno.h>
#include "vmelib.h"
#include "intreg.h"
#define INTREG_VECTOR INTREG_VECTOR0
#define LOOP 10
volatile struct intreg *mm;
extern errno;

int count;
void dummy() {}

main(argc, argv)
int argc;
char **argv;
{
    int i, j;
    volatile caddr_t vme_addr;
    int loop,bit;
    int status, intrno, intcount;
    struct sigvec vec;
    int pid;
    int pipeid[2];
    struct timeval timevalue0[LOOP], timevalue1[LOOP];
    int elapsed[LOOP];
    int average;

/* define signal handler for SIGALRM */
    vec.sv_handler = dummy;
    vec.sv_mask = 0;
    vec.sv_flags = 0;
#ifdef HPUX
    sigvector(SIGALRM, &vec, NULL);
#else
    sigvec(SIGALRM, &vec, NULL);
#endif
    count = 0;

    if( pipe(pipeid) < 0 ) {
      printf("pipe open error\n");
      exit(0);
    }

    vme_addr = (volatile caddr_t)vme_mapopen(VME_A16D16MODE, INTREG_ADDR,
                                             INTREG_SIZE);
    if (vme_addr == (volatile caddr_t)-1 ) {
        printf("intreg cannot open! \n");
        exit(1);
    }

    mm = (volatile struct intreg *)vme_addr;

    printf("address = %x\n", mm);

    /* display contents of registers... */
    printf("Latch 0 = %4X\n",mm->latch[0] & 0xFF);
    printf("Latch 1 = %4X\n",mm->latch[1] & 0xFF);
    printf("in      = %4X\n",mm->in & 0xFF);
    printf("csr 0   = %4X\n",mm->csr[0] & 0xFF);
    printf("csr 1   = %4X\n",mm->csr[1] & 0xFF);

    pid = fork();
    if( pid == 0 ) { /* Child process */
        /* Generate Interrupt Pulse */
        close(pipeid[0]);
        sleep(4);
        for( i = 0; i < LOOP; i++ ) {
            mm->csr[0] = 0x5b;
            mm->csr[1] = 0x5b;
	    get_time(&timevalue0[i]);
            mm->pulse = 0x0F0;
	    write( pipeid[1], &timevalue0[i], sizeof(struct timeval));
            sleep(1);
            mm->csr[0] = 1;
	}
	exit(0);
    } else if ( pid < 0 ) { /* Error */
        printf("Can't fork\n");
	exit(1);
    }
    /* Parent process */
    /* open interrupt vector */
    if( (status = vme_intopen(INTREG_VECTOR, &intrno)) != 0 ) {
        printf("errno = %d\n", status );
	exit(0);
    }
    printf("intrno = %d\n", intrno);
    printf("vme_intopen -- intrno = %d\n", intrno);
    printf("vme_intopen done\n");
    /* enable the vector */
    if( (status = vme_intenable(intrno) ) != 0 ) {
        printf("errno = %d\n", status );
        exit(0);
    }
    printf("vme_intenable done\n");
    /* wait for an interrupt */
    printf("interrupt count = %d\n", intcount);

    /* Generate Interrupt Pulse */
    close(pipeid[1]);
    for( i = 0; i < LOOP; i++ ) {
	if( (status = vme_intwait(intrno, &intcount, 10)) != 0 ) {
	    printf("errno = %d\n", status );
	}
	get_time(&timevalue1[i]);
	read(pipeid[0], &timevalue0[i], sizeof(struct timeval));
	errno = 0;
/*	printf("Parent process : intcount = %d\n", intcount);
	printf("Parent process : CSR0 = %3X  CSR1 = %3X\n",
	       mm->csr[0]&0x0FF,mm->csr[1]&0x0FF);
*/
    }
    average = 0;
    for(i = 0; i < LOOP; i++ ) {
      elapsed[i] = e_time(&timevalue0[i], &timevalue1[i]);
      average = average + elapsed[i];
    }
    printf("average of elapsed time is %8.1f\n", (float)average/(float)LOOP);
    /* close the vector */
    vme_intclose(intrno);
}

get_time(timevalue)
struct timeval *timevalue; {
  struct timezone tzp;

  gettimeofday (timevalue, &tzp);
}

int e_time(first, second)
struct timeval *first, *second; {

  if (first->tv_usec > second->tv_usec) {
    second->tv_usec += 1000000;
    second->tv_sec--;
  }
  printf("elapsed time %d sec, %d microsec\n",
	 second->tv_sec - first->tv_sec, second->tv_usec - first->tv_usec);
  return( (int)(second->tv_sec - first->tv_sec)*1000000
	 + (int)(second->tv_usec - first->tv_usec));
}
