/* File name     : vmesingle_measure1.c
 * Creation date : March 1994
 * Author        : Y.Yasu, Online group, Physics department, KEK
 */

#include <stdio.h>
#include <sys/file.h>
#include<sys/types.h>
#ifdef HPRT
#include <shmmap.h>
#else
#include<sys/mman.h>
#endif
#include<sys/time.h> 
#include "vmelib.h"
#define MEM_A32D32 0x11000000
#define MEM_ADDR MEM_A32D32
#define MEM_MODE VME_A32D32MODE
#define MEM_SIZE 200000
#define MEM_SIZE_LONG MEM_SIZE/4
#define LOOP 100
extern int errno;

char buf[MEM_SIZE], bufw[MEM_SIZE];
volatile char *mm;
int  buf_l[MEM_SIZE_LONG], bufw_l[MEM_SIZE_LONG];
volatile long *mm_long;
struct timeval timevalue, timevalue0[LOOP], timevalue1[LOOP];
int elapsed[LOOP];
int average;
int overhead[LOOP];

main()
{
    int i, j;
    long *buf_long = buf_l;
    volatile caddr_t vme_addr;

    for(i=0;i<MEM_SIZE;i++) {
      if(i%256) bufw[i] = i;
      else bufw[i] = 1;
      buf[i]=0;
    }
    buf_long[0] = 0xaaaaaaaa;
    for(i=1;i<MEM_SIZE_LONG;i++) bufw_l[i] = bufw_l[i-1] + 1;
    vme_addr = (volatile caddr_t)vme_mapopen(MEM_MODE, MEM_ADDR,
                                             MEM_SIZE);
    if (vme_addr == ( volatile caddr_t)-1) {
        printf("mp1 cannot open! \n");
        exit(1);
    }

    mm = (volatile char *)vme_addr;
    mm_long = (volatile long *)vme_addr;
/*
    printf("address = %x\n", mm_long);
*/
    for( i = 0; i < LOOP; i++ ) {
      get_time(&timevalue0[i]);
      for( j = 0; j < 1000; j++ ) {
	get_time(&timevalue);
      }
      get_time(&timevalue1[i]);
      overhead[i] = e_time(&timevalue0[i], &timevalue1[i]);
      overhead[i] /= 1000;
    }

    printf("VME access speed measurement with %d KB\n", MEM_SIZE/1000);

    printf("strncpy : from host to VME\n");
    for( i = 0; i < LOOP; i++) {
      get_time(&timevalue0[i]);
      strncpy(mm, bufw, MEM_SIZE);
      get_time(&timevalue1[i]);
      elapsed[i] = e_time(&timevalue0[i], &timevalue1[i]);
    }
    print_time();

    for(i=0;i<MEM_SIZE;i++) buf[i] = 0;
    printf("strncpy : from VME to host\n");
    for( i = 0; i < LOOP; i++) {
      get_time(&timevalue0[i]);
      strncpy(buf, mm, MEM_SIZE);
      get_time(&timevalue1[i]);
      elapsed[i] = e_time(&timevalue0[i], &timevalue1[i]);
    }
    if( !(compare_data( buf, bufw, MEM_SIZE) ) )
      print_time();

    printf("memcpy : from host to VME\n");
    for( i = 0; i<LOOP; i++) {
      get_time(&timevalue0[i]);
      memcpy(mm,bufw,MEM_SIZE); 
      get_time(&timevalue1[i]);
      elapsed[i] = e_time(&timevalue0[i], &timevalue1[i]);
    }
    print_time();

    for(i=0;i<MEM_SIZE;i++) buf[i] = 0;
    printf("memcpy : from VME to host\n");
    for( i = 0; i<LOOP; i++) {
      get_time(&timevalue0[i]);
      memcpy(buf,mm,MEM_SIZE) ; 
      get_time(&timevalue1[i]);
      elapsed[i] = e_time(&timevalue0[i], &timevalue1[i]);
    }
    if( !(compare_data( buf, bufw, MEM_SIZE) ) )
      print_time();

    printf("doloop : from host to VME\n");
    for( i = 0; i<LOOP; i++) {
      get_time(&timevalue0[i]);
      for( j = 0; j< MEM_SIZE_LONG; j++)
	mm_long[j] = (volatile int)bufw_l[j];
      get_time(&timevalue1[i]);
      elapsed[i] = e_time(&timevalue0[i], &timevalue1[i]);
    }
    print_time();

    for(i=0;i<MEM_SIZE_LONG;i++) buf_l[i] = 0;
    printf("doloop : from VME to host\n");
    for( i = 0; i<LOOP; i++) {
      get_time(&timevalue0[i]);
      for( j = 0; j<MEM_SIZE_LONG; j++)
	buf_l[j] = (long)mm_long[j];
      get_time(&timevalue1[i]);
      elapsed[i] = e_time(&timevalue0[i], &timevalue1[i]);
    }
    if( !(compare_data( buf_l, bufw_l, MEM_SIZE) ) )
      print_time();

}

get_time(timevalue)
struct timeval *timevalue; {
  struct timezone tzp;

  gettimeofday (timevalue, &tzp);
}

int e_time(first, second)
struct timeval *first, *second; {

  if (first->tv_usec > second->tv_usec) {
    second->tv_usec += 1000000;
    second->tv_sec--;
  }
/*  printf("elapsed time %d sec, %d microsec\n",
         second->tv_sec - first->tv_sec, second->tv_usec - first->tv_usec);
*/
  return( (int)(second->tv_sec - first->tv_sec)*1000000
         + (int)(second->tv_usec - first->tv_usec));
}

print_time() {
    float usec, size;
    int i;

    average = 0;
    for( i = 2; i < LOOP; i++ )
      average = average + elapsed[i] - overhead[i];
    printf("    average of elapsed time is %d microsec.\n",average/(LOOP-2));
    usec = (float)(average/(LOOP-2));
    size = (float)MEM_SIZE;
    printf("    speed of data transfer : %3.1f MB/sec\n", size/usec);
}

int compare_data( buf, bufw, size)
char *buf, *bufw;
int size;
{
  int i;
  for(i = 0; i < size; i++)
    if( buf[i] != bufw[i] ) {
      printf("    error at location %d\n", i);
      printf("    written data = %x  read data = %x\n", bufw[i], buf[i]);
      return(-1);
    }
  return 0;
}
