/* File name     : vs_check.c
 * Creation date : November 1994
 * Author        : Y.Yasu, Online group, Physics department, KEK
 * Description   : The program checks the function of VME Visual Scaler
 *                 produced by KEK. It makes pulses to V.S. and checks the
 *                 count by readding V.S. with the written count.
 *                 All channels of the V.S. should be connected to OUTPUTs
 *                 of a VME Interrupt register module.
 *                 vs_check loop_count period<return>
 *                     loop_count -- loop count to be executed
 *                     period     -- check count
 */

#include <stdio.h>
#include<sys/types.h>
#include "vmelib.h"
#include "vs.h"
#include "intreg.h"
#define INTREG_VECTOR INTREG_VECTOR0

union vs vsdata;
volatile union vs *vs;
union channel8data ch8;
volatile struct intreg *mm;

#if defined(ULTRIX) || defined(DEC)
long swapword();
#endif
int bcd2i();

main(argc, argv)
int argc;
char **argv;
{
    int i, j;
    int loop, error_count;
    volatile caddr_t vme_addr, mm_t;
    int period;

    vme_addr = (volatile caddr_t)vme_mapopen(VME_A24D16MODE, VS_ADDR,
                                             VS_SIZE);
    if (vme_addr == (volatile caddr_t)-1 ) {
        printf("visual scaler cannot open! \n");
        exit(1);
    }
    vs = (volatile union vs *)vme_addr;
    printf("address of Visual Scaler = %x\n", vs);
    

    vme_addr = (volatile caddr_t)vme_mapopen(VME_A16D16MODE, INTREG_ADDR,
                                             INTREG_SIZE);
    if (vme_addr == (volatile caddr_t)-1 ) {
        printf("intreg cannot open! \n");
        exit(1);
    }
    mm = (volatile struct intreg *)vme_addr;
    printf("address of Interrupt Register = %x\n", mm);

    if( argc < 2 ) loop = 10000;
    else loop = atoi(argv[1]);
    if( argc < 3 ) period = 100;
    else period = atoi(argv[2]);

    printf("csr0 of I.R. = %x\n", mm->csr[0]);

    vs->reg[0xFE/2] = 0xFF; /* Reset Visucal Scaler */
    mm->out = 0x0;
    error_count = 0;

    for( i = 0; i < loop; i++) {
      if( i%period == 0) {
      for( j = 0; j < 8; j++) {
	ch8.data[0] = vs->ch8[j].data[0];
	ch8.data[1] = vs->ch8[j].data[1];
#if defined(ULTRIX) || defined(DEC)
	swapword( &ch8.ldata );
#endif
	bcd2i( &ch8.ldata );
	if( ch8.ldata != i ) {
	  printf("Error : written pulse = %x read data = %x at ch. = %d\n",
		 i, ch8.ldata, j);
	  error_count++;
	  if( error_count > 10 ) exit(0);
	}
      }
    }
      mm->pulse = 0xFF;
      mm->csr[0] = 1;
    }
}

