/* File name     : vmelib.c
 * Creation date : September 1993
 * Author        : Y.Yasu, Online group, Physics department, KEK
 * Version       : 1.03
 * Modified date : August 1994, Y.Yasu
 *               : For HP-RT
 *               : Octover 1994, Y.Yasu
 *               : For fixing bug
 *               : November 1994, Y.Yasu
 *               : For fixing bug
 *               : November 1995, Y.Yasu
 *               : For cleaning codes of the interrupt routines
 */

#include	<stdio.h>
#include        <fcntl.h>
#include	<file.h>
#include        <shmmap.h>
#include	<sys/types.h>
#include	<sys/file.h>
#include        <sys/ioctl.h>
#include        <sys/errno.h>
#include        "vmelib.h"
#define         SLOT 3
/* SLOT = 0 for VME_A16, SLOT = 1 for VME_A24, SLOT = 2 for VME_A32 */
extern int      errno;

/* For VME devices */

static int vme_fd[SLOT];
static int vme_offset[SLOT];

/* For interrupt handler */
static int vme_path;    /* file descriptor for Interrupt handling */
static int vme_dmapath;    /* file descriptor for Block Transfer */
static struct vmecmd vmecmd;

caddr_t vme_mapopen(mode, vmeaddr, size)
int mode;
caddr_t vmeaddr;
size_t size;
{
    caddr_t vme_vaddr;	  /* virtual address */
    int slot;
    char vme_dev[15];
    int vme_mapsize;
    int vme_pagesize;

    vme_pagesize = getpagesize();

    switch (mode) {
    case VME_A16D32MODE :
    case VME_A16D16MODE :
#ifdef DEBUG
        printf("VME_A16MODE\n");
#endif
	slot = 0;
        if( size > VME_A16SIZE )
	    return (volatile caddr_t)-1;
	if( (vmeaddr < (volatile caddr_t)VME_A16BASE) || 
	    ((vmeaddr+size) > (caddr_t)(VME_A16BASE+VME_A16SIZE)) )
	    return (volatile caddr_t)-1;
        vme_offset[slot] = (int)vmeaddr&0xFFFF - VME_A16BASE;
        strcpy(vme_dev,"/dev/vmea16");
	break;
    case VME_A24D16MODE :
    case VME_A24D32MODE :
#ifdef DEBUG
        printf("VME_A24MODE\n");
#endif
	slot = 1;
        if( size > VME_A24SIZE )
	    return (volatile caddr_t)-1;
	if( (vmeaddr < (volatile caddr_t)VME_A24BASE) || 
	    ((vmeaddr+size) > (volatile caddr_t)(VME_A24BASE+VME_A24SIZE)) )
	    return (volatile caddr_t)-1;
        vme_offset[slot] = (int)vmeaddr&0xFFFFFF - VME_A24BASE;
        strcpy(vme_dev,"/dev/vmea24");
	break;
    case VME_A32D16MODE :
    case VME_A32D32MODE :
#ifdef DEBUG
        printf("VME_A32 MODE\n");
#endif
	slot = 2;
        if( size > VME_A32SIZE )
	    return (volatile caddr_t)-1;
	if( (vmeaddr < (volatile caddr_t)VME_A32BASE) || 
	    ((vmeaddr+size) > (volatile caddr_t)(VME_A32BASE+VME_A32SIZE)) )
	    return (volatile caddr_t)-1;
        vme_offset[slot] = (int)vmeaddr - VME_A32BASE;
        strcpy(vme_dev,"/dev/vmea32");
	break;
    };
    vme_mapsize = size + vme_offset[slot]%vme_pagesize;
    if ((vme_fd[slot] = open(vme_dev, O_RDWR)) == -1)
        return (volatile caddr_t)-1;
    /* if mmap fails, return value becomes -1 */
    vme_vaddr = (caddr_t)shmmap(vme_fd[slot], 0, vme_mapsize,
				vme_offset[slot]/vme_pagesize*vme_pagesize,
				SHM_READ | SHM_WRITE );
#ifdef DEBUG
    printf("vme_vaddr = %x\n", vme_vaddr);
    printf("vme_offset = %x\n", vme_offset[slot]);
    printf("vme_pagesize = %x\n", vme_pagesize);
#endif
    if (vme_vaddr == (volatile caddr_t) 0) {
#ifdef DEBUG
        printf("errno = %d\n", errno);
#endif
        return (volatile caddr_t)-1;
    } else
        return (vme_vaddr + (vme_offset[slot]%vme_pagesize));
}

caddr_t vme_mapclose(addr, mode)
caddr_t addr;
int mode; {
    int status, vme_vaddr, slot;
    int vme_pagesize;

    switch (mode) {
    case VME_A16D32MODE :
    case VME_A16D16MODE :
        slot = 0;
	break;
    case VME_A24D32MODE :
    case VME_A24D16MODE :
        slot = 1;
	break;
    case VME_A32D32MODE :
    case VME_A32D16MODE :
        slot = 2;
	break;
    }
    vme_vaddr = (int)addr - vme_offset[slot];
#ifdef DEBUG
    printf("shmunmap : unmap address = %x\n", vme_vaddr);
#endif
    status = shmunmap((void *)vme_vaddr, (size_t)0);
    if( status == -1 ) {
#ifdef DEBUG
      printf("shmunmap error = %d\n", errno);
#endif
      return (caddr_t)-1;
    }
    close(vme_fd[slot]);
    return 0;
}

int vme_intopen( vector, intrno )
int vector;
int *intrno; {
    int status;

    errno = 0;
    if ((vme_path = open("/dev/int", O_RDWR)) == -1)
        return errno;
    vmecmd.intrno = vector;
#ifdef DEBUG
    printf("vme_intopen: open ok\n");
    printf("vme_intopen: vector num = %x\n", vector);
#endif
    if( (status = ioctl(vme_path, VMEIOC_INTOPEN, &vmecmd )) != 0 )
        return status;
    *intrno = vmecmd.data;
    return 0;
}

int vme_intclose( intrno )
int intrno; {
    int status;

    vmecmd.intrno = intrno;
    errno = 0;
    if( ( status = ioctl( vme_path, VMEIOC_INTCLOSE, &intrno ) ) != 0 )
        return status;
    close(vme_path);
    return 0;
}

int vme_intenable( intrno )
int intrno; {
    int intrno1 = intrno;
    int status;

    errno = 0;
    if( ( status = ioctl( vme_path, VMEIOC_INTENABLE, &intrno1 ) ) != 0 )
        return status;
    return 0;
}

int vme_intdisable( intrno )
int intrno; {
    int status;

    if( ( status = ioctl( vme_path, VMEIOC_INTDISABLE, &intrno ) ) != 0 )
        return status;
    return 0;
}

int vme_intwait( intrno, interrupt_count, timeout )
int intrno, *interrupt_count, timeout; {
    int status;

    vmecmd.intrno = intrno;
    vmecmd.data = timeout;
    status = ioctl( vme_path, VMEIOC_INTWAIT, &vmecmd );
#ifdef DEBUG
    printf("vme_intwait:vmecmd.reply1 = %d\n", vmecmd.reply1);
    printf("vme_intwait:vmecmd.reply2 = %d\n", vmecmd.reply2);
#endif
    *interrupt_count = vmecmd.reply1;
    if( vmecmd.reply2 == -1 )
        return ETIMEDOUT;
    return status;
}

int vme_intnowait( intrno, interrupt_count )
int intrno, *interrupt_count; {
    int status;

    vmecmd.intrno = intrno;
    status = ioctl( vme_path, VMEIOC_INTNOWAIT, &vmecmd );
#ifdef DEBUG
    printf("vme_intnowait:vmecmd.reply1 = %d\n", vmecmd.reply1);
    printf("vme_intnowait:vmecmd.reply2 = %d\n", vmecmd.reply2);
#endif
    *interrupt_count = vmecmd.reply1;
    if( errno == EINTR && vmecmd.reply2 == 0 )
        return 0;
    return status;
}

int vme_intclear( intrno )
int intrno; {

    return ( ioctl( vme_path, VMEIOC_INTCLEAR, &intrno ));
}

int vme_intcheck( intrno, interrupt_count )
int intrno, *interrupt_count; {
    int status;

    vmecmd.intrno = intrno;
    status = ioctl(vme_path, VMEIOC_INTCHECK, &vmecmd );
#ifdef DEBUG
    printf("vme_intcheck:vmecmd.reply1 = %d\n", vmecmd.reply1);
    printf("vme_intcheck;vmecmd.reply2 = %d\n", vmecmd.reply2);
#endif
    *interrupt_count = vmecmd.reply1;
    return status;
}
